@@ -132,9 +132,14 @@ extern "C" {
  *			  QB_LOG_FILTER_FORMAT, "ringbuffer", LOG_TRACE);
  * @endcode
  *
- * @par Threaded logging.
- * To achieve non-blocking logging you can use threaded logging. So any
- * calls to write() or syslog() will not hold up your program.
+ * @par Thread safe non-blocking logging.
+ * Logging is only thread safe when threaded logging is in use. If you plan
+ * on logging from multiple threads, you must initialize libqb's logger thread
+ * and use qg_log_filter_ctl to set the QB_LOG_CONF_THREADED flag on all the
+ * logging targets in use.
+ *
+ * To achieve non-blocking logging you can use threaded logging as well
+ * So any calls to write() or syslog() will not hold up your program.
  *
  * Threaded logging use:
  * @code
@@ -358,18 +363,30 @@ void qb_log_from_external_source_va(const char *function,
  */
 #define qb_log(priority, fmt, args...) qb_logt(priority, 0, fmt, ##args)
 
+/* Define the character used to mark the beginning of "extended" information;
+ * a string equivalent is also defined so clients can use it like:
+ *    qb_log(level, "blah blah "QB_XS" yada yada", __func__);
+ */
+#define QB_XC '\a'
+#define QB_XS "\a"
+
 /**
  * This is similar to perror except it goes into the logging system.
  *
  * @param priority this takes syslog priorities.
  * @param fmt usual printf style format specifiers
  * @param args usual printf style args
+ *
+ * @note Because qb_perror() adds the system error message and error number onto
+ *       the end of the given fmt, that information will become extended
+ *       information if QB_XS is used inside fmt and will not show up in any
+ *       logs that strip extended information.
  */
 #ifndef S_SPLINT_S
 #define qb_perror(priority, fmt, args...) do {				\
 	char _perr_buf_[QB_LOG_STRERROR_MAX_LEN];			\
 	const char *_perr_str_ = qb_strerror_r(errno, _perr_buf_, sizeof(_perr_buf_));	\
-	qb_logt(priority, 0, fmt ": %s (%d)", ##args, _perr_str_, errno);		\
+	qb_logt(priority, 0, fmt ": %s (%d)", ##args, _perr_str_, errno); \
     } while(0)
 #else
 #define qb_perror
@@ -400,12 +417,16 @@ enum qb_log_conf {
 	QB_LOG_CONF_PRIORITY_BUMP,
 	QB_LOG_CONF_STATE_GET,
 	QB_LOG_CONF_FILE_SYNC,
+	QB_LOG_CONF_EXTENDED,
 };
 
 enum qb_log_filter_type {
 	QB_LOG_FILTER_FILE,
 	QB_LOG_FILTER_FUNCTION,
 	QB_LOG_FILTER_FORMAT,
+	QB_LOG_FILTER_FILE_REGEX,
+	QB_LOG_FILTER_FUNCTION_REGEX,
+	QB_LOG_FILTER_FORMAT_REGEX,
 };
 
 enum qb_log_filter_conf {
