@@ -608,6 +608,10 @@ qb_ipcs_disconnect(struct qb_ipcs_connection *c)
 		c->service->funcs.disconnect(c);
 		c->state = QB_IPCS_CONNECTION_INACTIVE;
 		c->service->stats.closed_connections++;
+
+		/* This removes the initial alloc ref */
+		qb_ipcs_connection_unref(c);
+
 		/* return early as it's an incomplete connection.
 		 */
 		return;
@@ -666,8 +670,6 @@ _process_request_(struct qb_ipcs_connection *c, int32_t ms_timeout)
 	ssize_t size;
 	struct qb_ipc_request_header *hdr;
 
-	qb_ipcs_connection_ref(c);
-
 	if (c->service->funcs.peek && c->service->funcs.reclaim) {
 		size = c->service->funcs.peek(&c->request, (void **)&hdr,
 					      ms_timeout);
@@ -709,7 +711,6 @@ _process_request_(struct qb_ipcs_connection *c, int32_t ms_timeout)
 	}
 
 cleanup:
-	qb_ipcs_connection_unref(c);
 	return res;
 }
 
@@ -941,3 +942,24 @@ qb_ipcs_connection_auth_set(qb_ipcs_connection_t *c, uid_t uid,
 		c->auth.mode = mode;
 	}
 }
+
+int32_t
+qb_ipcs_connection_get_buffer_size(qb_ipcs_connection_t *c)
+{
+	if (c == NULL) {
+		return -EINVAL;
+	}
+
+	/* request, response, and event shoud all have the same
+	 * buffer size allocated. It doesn't matter which we return
+	 * here. */
+	return c->response.max_msg_size;
+}
+
+void qb_ipcs_enforce_buffer_size(qb_ipcs_service_t *s, uint32_t buf_size)
+{
+	if (s == NULL) {
+		return;
+	}
+	s->max_buffer_size = buf_size;
+}
