@@ -112,7 +112,7 @@ do {							\
 } while (0)
 
 static void print_header(struct qb_ringbuffer_s * rb);
-static void _rb_chunk_reclaim(struct qb_ringbuffer_s * rb);
+static int _rb_chunk_reclaim(struct qb_ringbuffer_s * rb);
 
 qb_ringbuffer_t *
 qb_rb_open(const char *name, size_t size, uint32_t flags,
@@ -138,7 +138,9 @@ qb_rb_open_2(const char *name, size_t size, uint32_t flags,
 	void *shm_addr;
 	long page_size = sysconf(_SC_PAGESIZE);
 
-#ifdef QB_FORCE_SHM_ALIGN
+#ifdef QB_ARCH_HPPA
+	page_size = QB_MAX(page_size, 0x00400000); /* align to page colour */
+#elif defined(QB_FORCE_SHM_ALIGN)
 	page_size = QB_MAX(page_size, 16 * 1024);
 #endif /* QB_FORCE_SHM_ALIGN */
 	/* The user of this api expects the 'size' parameter passed into this function
@@ -232,7 +234,7 @@ qb_rb_open_2(const char *name, size_t size, uint32_t flags,
 	}
 
 	qb_util_log(LOG_DEBUG,
-		    "shm size:%zd; real_size:%zd; rb->word_size:%d", size,
+		    "shm size:%ld; real_size:%ld; rb->word_size:%d", size,
 		    real_size, rb->shared_hdr->word_size);
 
 	/* this function closes fd_data */
@@ -449,7 +451,11 @@ qb_rb_chunk_alloc(struct qb_ringbuffer_s * rb, size_t len)
 	 */
 	if (rb->flags & QB_RB_FLAG_OVERWRITE) {
 		while (qb_rb_space_free(rb) < (len + QB_RB_CHUNK_MARGIN)) {
-			_rb_chunk_reclaim(rb);
+			int rc = _rb_chunk_reclaim(rb);
+			if (rc != 0) {
+				errno = rc;
+				return NULL;
+			}
 		}
 	} else {
 		if (qb_rb_space_free(rb) < (len + QB_RB_CHUNK_MARGIN)) {
@@ -555,18 +561,19 @@ qb_rb_chunk_write(struct qb_ringbuffer_s * rb, const void *data, size_t len)
 	return len;
 }
 
-static void
+static int
 _rb_chunk_reclaim(struct qb_ringbuffer_s * rb)
 {
 	uint32_t old_read_pt;
 	uint32_t new_read_pt;
 	uint32_t old_chunk_size;
 	uint32_t chunk_magic;
+	int rc = 0;
 
 	old_read_pt = rb->shared_hdr->read_pt;
 	chunk_magic = QB_RB_CHUNK_MAGIC_GET(rb, old_read_pt);
 	if (chunk_magic != QB_RB_CHUNK_MAGIC) {
-		return;
+		return -EINVAL;
 	}
 
 	old_chunk_size = QB_RB_CHUNK_SIZE_GET(rb, old_read_pt);
@@ -587,7 +594,7 @@ _rb_chunk_reclaim(struct qb_ringbuffer_s * rb)
 	rb->shared_hdr->read_pt = new_read_pt;
 
 	if (rb->notifier.reclaim_fn) {
-		int rc = rb->notifier.reclaim_fn(rb->notifier.instance,
+		rc = rb->notifier.reclaim_fn(rb->notifier.instance,
 						 old_chunk_size);
 		if (rc < 0) {
 			errno = -rc;
@@ -601,6 +608,8 @@ _rb_chunk_reclaim(struct qb_ringbuffer_s * rb)
 		     old_read_pt,
 		     rb->shared_hdr->read_pt,
 		     rb->shared_hdr->write_pt);
+
+	return rc;
 }
 
 void
@@ -879,7 +888,11 @@ qb_rb_create_from_file(int32_t fd, uint32_t flags)
 	 * 6. data
 	 */
 	n_required = (word_size * sizeof(uint32_t));
-	rb = qb_rb_open("create_from_file", n_required,
+
+	/*
+	 * qb_rb_open adds QB_RB_CHUNK_MARGIN + 1 to the requested size.
+	 */
+	rb = qb_rb_open("create_from_file", n_required - (QB_RB_CHUNK_MARGIN + 1),
 			QB_RB_FLAG_CREATE | QB_RB_FLAG_NO_SEMAPHORE, 0);
 	if (rb == NULL) {
 		return NULL;
