@@ -32,7 +32,7 @@
 #include <qb/qbipcs.h>
 #include <qb/qbloop.h>
 
-static const char *ipc_name = "ipc_test";
+static char ipc_name[256];
 
 #define DEFAULT_MAX_MSG_SIZE (8192*16)
 static int CALCULATED_DGRAM_MAX_MSG_SIZE = 0;
@@ -48,6 +48,7 @@ static int CALCULATED_DGRAM_MAX_MSG_SIZE = 0;
  * this the largests msg we can successfully send. */
 #define GIANT_MSG_DATA_SIZE MAX_MSG_SIZE - sizeof(struct qb_ipc_response_header) - 8
 
+static int enforce_server_buffer=0;
 static qb_ipcc_connection_t *conn;
 static enum qb_ipc_type ipc_type;
 
@@ -93,6 +94,7 @@ static int32_t disconnect_after_created = QB_FALSE;
 static int32_t num_bulk_events = 10;
 static int32_t num_stress_events = 30000;
 static int32_t reference_count_test = QB_FALSE;
+static int32_t multiple_connections = QB_FALSE;
 
 
 static int32_t
@@ -103,6 +105,18 @@ exit_handler(int32_t rsignal, void *data)
 	return -1;
 }
 
+static void
+set_ipc_name(const char *prefix)
+{
+	/* We have to give the server name a random postfix because
+	 * some build systems attempt to generate packages for libqb
+	 * in parallel. These unit tests are run during the package
+	 * build process. Two builds executing on the same machine
+	 * can stomp on each other's unit tests if the ipc server
+	 * names aren't unique... This was very confusing to debug */
+	snprintf(ipc_name, 256, "%s-%d", prefix, (int32_t)random());
+}
+
 static int32_t
 s1_msg_process_fn(qb_ipcs_connection_t *c,
 		void *data, size_t size)
@@ -138,6 +152,7 @@ s1_msg_process_fn(qb_ipcs_connection_t *c,
 		int32_t m;
 		int32_t num;
 		struct qb_ipcs_connection_stats_2 *stats;
+		uint32_t max_size = MAX_MSG_SIZE;
 
 		response.size = sizeof(struct qb_ipc_response_header);
 		response.error = 0;
@@ -147,8 +162,7 @@ s1_msg_process_fn(qb_ipcs_connection_t *c,
 		free(stats);
 
 		/* crazy large message */
-		res = qb_ipcs_event_send(c, &response,
-					 MAX_MSG_SIZE*10);
+		res = qb_ipcs_event_send(c, &response, max_size*10);
 		ck_assert_int_eq(res, -EMSGSIZE);
 
 		/* send one event before responding */
@@ -156,6 +170,11 @@ s1_msg_process_fn(qb_ipcs_connection_t *c,
 		ck_assert_int_eq(res, sizeof(response));
 		response.id++;
 
+		/* There should be one more item in the event queue now. */
+		stats = qb_ipcs_connection_stats_get_2(c, QB_FALSE);
+		ck_assert_int_eq(stats->event_q_length - num, 1);
+		free(stats);
+
 		/* send response */
 		response.id = IPC_MSG_RES_BULK_EVENTS;
 		res = qb_ipcs_response_send(c, &response, response.size);
@@ -174,9 +193,6 @@ s1_msg_process_fn(qb_ipcs_connection_t *c,
 			ck_assert_int_eq(res, sizeof(response));
 			response.id++;
 		}
-		stats = qb_ipcs_connection_stats_get_2(c, QB_FALSE);
-		ck_assert_int_eq(stats->event_q_length - num, num_bulk_events);
-		free(stats);
 
 	} else if (req_pt->id == IPC_MSG_REQ_STRESS_EVENT) {
 		struct {
@@ -224,6 +240,7 @@ s1_msg_process_fn(qb_ipcs_connection_t *c,
 	} else if (req_pt->id == IPC_MSG_REQ_SERVER_FAIL) {
 		exit(0);
 	} else if (req_pt->id == IPC_MSG_REQ_SERVER_DISCONNECT) {
+		multiple_connections = QB_FALSE;
 		qb_ipcs_disconnect(c);
 	}
 	return 0;
@@ -260,6 +277,9 @@ my_dispatch_del(int32_t fd)
 static int32_t
 s1_connection_closed(qb_ipcs_connection_t *c)
 {
+	if (multiple_connections) {
+		return 0;
+	}
 	qb_enter();
 	qb_leave();
 	return 0;
@@ -298,6 +318,10 @@ outq_flush (void *data)
 static void
 s1_connection_destroyed(qb_ipcs_connection_t *c)
 {
+	if (multiple_connections) {
+		return;
+	}
+
 	qb_enter();
 	if (reference_count_test) {
 		struct cs_ipcs_conn_context *cnx;
@@ -312,6 +336,11 @@ s1_connection_destroyed(qb_ipcs_connection_t *c)
 static void
 s1_connection_created(qb_ipcs_connection_t *c)
 {
+	uint32_t max = MAX_MSG_SIZE;
+	if (multiple_connections) {
+		return;
+	}
+
 	if (send_event_on_created) {
 		struct qb_ipc_response_header response;
 		int32_t res;
@@ -333,6 +362,10 @@ s1_connection_created(qb_ipcs_connection_t *c)
 		memcpy(context, "test", 4);
 		qb_ipcs_context_set(c, context);
 	}
+
+
+	ck_assert_int_eq(max, qb_ipcs_connection_get_buffer_size(c));
+
 }
 
 static void
@@ -355,6 +388,7 @@ run_ipc_server(void)
 		.dispatch_mod = my_dispatch_mod,
 		.dispatch_del = my_dispatch_del,
 	};
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	qb_loop_signal_add(my_loop, QB_LOOP_HIGH, SIGSTOP,
 			   NULL, exit_handler, &handle);
@@ -366,6 +400,9 @@ run_ipc_server(void)
 	s1 = qb_ipcs_create(ipc_name, 4, ipc_type, &sh);
 	fail_if(s1 == 0);
 
+	if (enforce_server_buffer) {
+		qb_ipcs_enforce_buffer_size(s1, max_size);
+	}
 	qb_ipcs_poll_handlers_set(s1, &ph);
 
 	res = qb_ipcs_run(s1);
@@ -392,13 +429,70 @@ run_function_in_new_process(void (*run_ipc_server_fn)(void))
 	return pid;
 }
 
-static int32_t
-stop_process(pid_t pid)
+static void
+request_server_exit(void)
+{
+	struct qb_ipc_request_header req_header;
+	struct qb_ipc_response_header res_header;
+	struct iovec iov[1];
+	int32_t res;
+
+	/*
+	 * tell the server to exit
+	 */
+	req_header.id = IPC_MSG_REQ_SERVER_FAIL;
+	req_header.size = sizeof(struct qb_ipc_request_header);
+
+	iov[0].iov_len = req_header.size;
+	iov[0].iov_base = &req_header;
+
+	ck_assert_int_eq(QB_TRUE, qb_ipcc_is_connected(conn));
+
+	res = qb_ipcc_sendv_recv(conn, iov, 1,
+				 &res_header,
+				 sizeof(struct qb_ipc_response_header), -1);
+	/*
+	 * confirm we get -ENOTCONN or ECONNRESET
+	 */
+	if (res != -ECONNRESET && res != -ENOTCONN) {
+		qb_log(LOG_ERR, "id:%d size:%d", res_header.id, res_header.size);
+		ck_assert_int_eq(res, -ENOTCONN);
+	}
+}
+
+static void
+kill_server(pid_t pid)
 {
-	/* wait a bit for the server to shutdown by it's self */
-	usleep(100000);
 	kill(pid, SIGTERM);
 	waitpid(pid, NULL, 0);
+}
+
+static int32_t
+verify_graceful_stop(pid_t pid)
+{
+	int wait_rc = 0;
+	int status = 0;
+	int rc = 0;
+	int tries;
+
+	/* We need the server to be able to exit by itself */
+	for (tries = 10;  tries >= 0; tries--) {
+		sleep(1);
+		wait_rc = waitpid(pid, &status, WNOHANG);
+		if (wait_rc > 0) {
+			break;
+		}
+	}
+
+	ck_assert_int_eq(wait_rc, pid);
+	rc = WIFEXITED(status);
+	if (rc) {
+		rc = WEXITSTATUS(status);
+		ck_assert_int_eq(rc, 0);
+	} else {
+		fail_if(rc == 0);
+	}
+	
 	return 0;
 }
 
@@ -415,6 +509,7 @@ send_and_check(int32_t req_id, uint32_t size,
 	struct qb_ipc_response_header res_header;
 	int32_t res;
 	int32_t try_times = 0;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	request.hdr.id = req_id;
 	request.hdr.size = sizeof(struct qb_ipc_request_header) + size;
@@ -422,10 +517,9 @@ send_and_check(int32_t req_id, uint32_t size,
 	/* check that we can't send a message that is too big
 	 * and we get the right return code.
 	 */
-	res = qb_ipcc_send(conn, &request, MAX_MSG_SIZE*2);
+	res = qb_ipcc_send(conn, &request, max_size*2);
 	ck_assert_int_eq(res, -EMSGSIZE);
 
-
 repeat_send:
 	res = qb_ipcc_send(conn, &request, request.hdr.size);
 	try_times++;
@@ -467,6 +561,62 @@ repeat_send:
 	return res;
 }
 
+static void
+test_ipc_txrx_timeout(void)
+{
+	struct qb_ipc_request_header req_header;
+	struct qb_ipc_response_header res_header;
+	struct iovec iov[1];
+	int32_t res;
+	int32_t c = 0;
+	int32_t j = 0;
+	pid_t pid;
+	uint32_t max_size = MAX_MSG_SIZE;
+
+	pid = run_function_in_new_process(run_ipc_server);
+	fail_if(pid == -1);
+	sleep(1);
+
+	do {
+		conn = qb_ipcc_connect(ipc_name, max_size);
+		if (conn == NULL) {
+			j = waitpid(pid, NULL, WNOHANG);
+			ck_assert_int_eq(j, 0);
+			sleep(1);
+			c++;
+		}
+	} while (conn == NULL && c < 5);
+	fail_if(conn == NULL);
+
+	/* The dispatch response will only come over
+	 * the event channel, we want to verify the receive times
+	 * out when an event is returned with no response */
+	req_header.id = IPC_MSG_REQ_DISPATCH;
+	req_header.size = sizeof(struct qb_ipc_request_header);
+
+	iov[0].iov_len = req_header.size;
+	iov[0].iov_base = &req_header;
+
+	res = qb_ipcc_sendv_recv(conn, iov, 1,
+				 &res_header,
+				 sizeof(struct qb_ipc_response_header), 5000);
+
+	ck_assert_int_eq(res, -ETIMEDOUT);
+
+	request_server_exit();
+	verify_graceful_stop(pid);
+
+	/*
+	 * wait a bit for the server to die.
+	 */
+	sleep(1);
+
+	/*
+	 * this needs to free up the shared mem
+	 */
+	qb_ipcc_disconnect(conn);
+}
+
 static int32_t recv_timeout = -1;
 static void
 test_ipc_txrx(void)
@@ -475,13 +625,14 @@ test_ipc_txrx(void)
 	int32_t c = 0;
 	size_t size;
 	pid_t pid;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	pid = run_function_in_new_process(run_ipc_server);
 	fail_if(pid == -1);
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -494,7 +645,7 @@ test_ipc_txrx(void)
 	size = QB_MIN(sizeof(struct qb_ipc_request_header), 64);
 	for (j = 1; j < 19; j++) {
 		size *= 2;
-		if (size >= MAX_MSG_SIZE)
+		if (size >= max_size)
 			break;
 		if (send_and_check(IPC_MSG_REQ_TX_RX, size,
 				   recv_timeout, QB_TRUE) < 0) {
@@ -502,10 +653,17 @@ test_ipc_txrx(void)
 		}
 	}
 	if (turn_on_fc) {
+		/* can't signal server to shutdown if flow control is on */
 		ck_assert_int_eq(fc_enabled, QB_TRUE);
+		qb_ipcc_disconnect(conn);
+		/* TODO - figure out why this sleep is necessary */
+		sleep(1);
+		kill_server(pid);
+	} else {
+		request_server_exit();
+		qb_ipcc_disconnect(conn);
+		verify_graceful_stop(pid);
 	}
-	qb_ipcc_disconnect(conn);
-	stop_process(pid);
 }
 
 static void
@@ -518,13 +676,14 @@ test_ipc_exit(void)
 	int32_t c = 0;
 	int32_t j = 0;
 	pid_t pid;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	pid = run_function_in_new_process(run_ipc_server);
 	fail_if(pid == -1);
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -545,8 +704,8 @@ test_ipc_exit(void)
 				 sizeof(struct qb_ipc_response_header), -1);
 	ck_assert_int_eq(res, sizeof(struct qb_ipc_response_header));
 
-	/* kill the server */
-	stop_process(pid);
+	request_server_exit();
+	verify_graceful_stop(pid);
 
 	/*
 	 * wait a bit for the server to die.
@@ -563,7 +722,7 @@ START_TEST(test_ipc_exit_us)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	recv_timeout = 5000;
 	test_ipc_exit();
 	qb_leave();
@@ -574,19 +733,39 @@ START_TEST(test_ipc_exit_shm)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	recv_timeout = 1000;
 	test_ipc_exit();
 	qb_leave();
 }
 END_TEST
 
+START_TEST(test_ipc_txrx_shm_timeout)
+{
+	qb_enter();
+	ipc_type = QB_IPC_SHM;
+	set_ipc_name(__func__);
+	test_ipc_txrx_timeout();
+	qb_leave();
+}
+END_TEST
+
+START_TEST(test_ipc_txrx_us_timeout)
+{
+	qb_enter();
+	ipc_type = QB_IPC_SOCKET;
+	set_ipc_name(__func__);
+	test_ipc_txrx_timeout();
+	qb_leave();
+}
+END_TEST
+
 START_TEST(test_ipc_txrx_shm_tmo)
 {
 	qb_enter();
 	turn_on_fc = QB_FALSE;
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	recv_timeout = 1000;
 	test_ipc_txrx();
 	qb_leave();
@@ -598,7 +777,7 @@ START_TEST(test_ipc_txrx_shm_block)
 	qb_enter();
 	turn_on_fc = QB_FALSE;
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	recv_timeout = -1;
 	test_ipc_txrx();
 	qb_leave();
@@ -611,7 +790,7 @@ START_TEST(test_ipc_fc_shm)
 	turn_on_fc = QB_TRUE;
 	ipc_type = QB_IPC_SHM;
 	recv_timeout = 500;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_txrx();
 	qb_leave();
 }
@@ -622,7 +801,7 @@ START_TEST(test_ipc_txrx_us_block)
 	qb_enter();
 	turn_on_fc = QB_FALSE;
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	recv_timeout = -1;
 	test_ipc_txrx();
 	qb_leave();
@@ -634,7 +813,7 @@ START_TEST(test_ipc_txrx_us_tmo)
 	qb_enter();
 	turn_on_fc = QB_FALSE;
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	recv_timeout = 1000;
 	test_ipc_txrx();
 	qb_leave();
@@ -647,7 +826,7 @@ START_TEST(test_ipc_fc_us)
 	turn_on_fc = QB_TRUE;
 	ipc_type = QB_IPC_SOCKET;
 	recv_timeout = 500;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_txrx();
 	qb_leave();
 }
@@ -665,13 +844,14 @@ test_ipc_dispatch(void)
 	int32_t c = 0;
 	pid_t pid;
 	int32_t size;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	pid = run_function_in_new_process(run_ipc_server);
 	fail_if(pid == -1);
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -684,7 +864,7 @@ test_ipc_dispatch(void)
 	size = QB_MIN(sizeof(struct qb_ipc_request_header), 64);
 	for (j = 1; j < 19; j++) {
 		size *= 2;
-		if (size >= MAX_MSG_SIZE)
+		if (size >= max_size)
 			break;
 		if (send_and_check(IPC_MSG_REQ_DISPATCH, size,
 				   recv_timeout, QB_TRUE) < 0) {
@@ -692,15 +872,16 @@ test_ipc_dispatch(void)
 		}
 	}
 
+	request_server_exit();
 	qb_ipcc_disconnect(conn);
-	stop_process(pid);
+	verify_graceful_stop(pid);
 }
 
 START_TEST(test_ipc_disp_us)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_dispatch();
 	qb_leave();
 }
@@ -730,9 +911,16 @@ count_stress_events(int32_t fd, int32_t revents, void *data)
 			if (res != sizeof(giant_event_recv)) {
 				qb_log(LOG_DEBUG, "Unexpected recv size, expected %d got %d",
 					res, sizeof(giant_event_recv));
+
+				ck_assert_int_eq(res, sizeof(giant_event_recv));
 			} else if (giant_event_recv.sent_msgs != events_received) {
 				qb_log(LOG_DEBUG, "Server event mismatch. Server thinks we got %d msgs, but we only received %d",
 					giant_event_recv.sent_msgs, events_received);
+				/* This indicates that data corruption is occurring. Since the events
+				 * received is placed at the end of the giant msg, it is possible
+				 * that buffers were not allocated correctly resulting in us
+				 * reading/writing to uninitialized memeory at some point. */
+				ck_assert_int_eq(giant_event_recv.sent_msgs, events_received);
 			}
 		}
 	} else if (res != -EAGAIN) {
@@ -769,25 +957,77 @@ count_bulk_events(int32_t fd, int32_t revents, void *data)
 	return 0;
 }
 
+static void
+test_ipc_stress_connections(void)
+{
+	int32_t c = 0;
+	int32_t j = 0;
+	uint32_t max_size = MAX_MSG_SIZE;
+	int32_t connections = 0;
+	pid_t pid;
+
+	multiple_connections = QB_TRUE;
+
+	qb_log_filter_ctl(QB_LOG_STDERR, QB_LOG_FILTER_CLEAR_ALL,
+			  QB_LOG_FILTER_FILE, "*", LOG_TRACE);
+	qb_log_filter_ctl(QB_LOG_STDERR, QB_LOG_FILTER_ADD,
+			  QB_LOG_FILTER_FILE, "*", LOG_INFO);
+	qb_log_ctl(QB_LOG_STDERR, QB_LOG_CONF_ENABLED, QB_TRUE);
+
+	pid = run_function_in_new_process(run_ipc_server);
+	fail_if(pid == -1);
+	sleep(1);
+
+	for (connections = 1; connections < 70000; connections++) {
+		if (conn) {
+			qb_ipcc_disconnect(conn);
+			conn = NULL;
+		}
+		do {
+			conn = qb_ipcc_connect(ipc_name, max_size);
+			if (conn == NULL) {
+				j = waitpid(pid, NULL, WNOHANG);
+				ck_assert_int_eq(j, 0);
+				sleep(1);
+				c++;
+			}
+		} while (conn == NULL && c < 5);
+		fail_if(conn == NULL);
+		
+		if (((connections+1) % 1000) == 0) {
+			qb_log(LOG_INFO, "%d ipc connections made", connections+1);
+		}
+	}
+	multiple_connections = QB_FALSE;
+
+	request_server_exit();
+	verify_graceful_stop(pid);
+	qb_ipcc_disconnect(conn);
+
+	qb_log_filter_ctl(QB_LOG_STDERR, QB_LOG_FILTER_CLEAR_ALL,
+			  QB_LOG_FILTER_FILE, "*", LOG_TRACE);
+	qb_log_filter_ctl(QB_LOG_STDERR, QB_LOG_FILTER_ADD,
+			  QB_LOG_FILTER_FILE, "*", LOG_TRACE);
+	qb_log_ctl(QB_LOG_STDERR, QB_LOG_CONF_ENABLED, QB_TRUE);
+}
+
 static void
 test_ipc_bulk_events(void)
 {
-	struct qb_ipc_request_header req_header;
-	struct qb_ipc_response_header res_header;
-	struct iovec iov[1];
 	int32_t c = 0;
 	int32_t j = 0;
 	pid_t pid;
 	int32_t res;
 	qb_loop_t *cl;
 	int32_t fd;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	pid = run_function_in_new_process(run_ipc_server);
 	fail_if(pid == -1);
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -814,27 +1054,20 @@ test_ipc_bulk_events(void)
 	qb_loop_run(cl);
 	ck_assert_int_eq(events_received, num_bulk_events);
 
-	req_header.id = IPC_MSG_REQ_SERVER_FAIL;
-	req_header.size = sizeof(struct qb_ipc_request_header);
-
-	iov[0].iov_len = req_header.size;
-	iov[0].iov_base = &req_header;
-	res = qb_ipcc_sendv_recv(conn, iov, 1,
-				 &res_header,
-				 sizeof(struct qb_ipc_response_header), -1);
-	if (res != -ECONNRESET && res != -ENOTCONN) {
-		qb_log(LOG_ERR, "id:%d size:%d", res_header.id, res_header.size);
-		ck_assert_int_eq(res, -ENOTCONN);
-	}
-
+	request_server_exit();
 	qb_ipcc_disconnect(conn);
-	stop_process(pid);
+	verify_graceful_stop(pid);
 }
 
 static void
 test_ipc_stress_test(void)
 {
-	struct qb_ipc_request_header req_header;
+	struct {
+		struct qb_ipc_request_header hdr __attribute__ ((aligned(8)));
+		char data[GIANT_MSG_DATA_SIZE] __attribute__ ((aligned(8)));
+		uint32_t sent_msgs __attribute__ ((aligned(8)));
+	} __attribute__ ((aligned(8))) giant_req;
+
 	struct qb_ipc_response_header res_header;
 	struct iovec iov[1];
 	int32_t c = 0;
@@ -843,13 +1076,24 @@ test_ipc_stress_test(void)
 	int32_t res;
 	qb_loop_t *cl;
 	int32_t fd;
+	uint32_t max_size = MAX_MSG_SIZE;
+	/* This looks strange, but it serves an important purpose.
+	 * This test forces the server to enforce the MAX_MSG_SIZE
+	 * limit from the server side, which overrides the client's
+	 * buffer limit.  To verify this functionality is working
+	 * we set the client limit lower than what the server
+	 * is enforcing. */
+	int32_t client_buf_size = max_size - 1024;
+	int32_t real_buf_size;
 
+	enforce_server_buffer = 1;
 	pid = run_function_in_new_process(run_ipc_server);
+	enforce_server_buffer = 0;
 	fail_if(pid == -1);
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, client_buf_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -859,6 +1103,9 @@ test_ipc_stress_test(void)
 	} while (conn == NULL && c < 5);
 	fail_if(conn == NULL);
 
+	real_buf_size = qb_ipcc_get_buffer_size(conn);
+	ck_assert_int_eq(real_buf_size, max_size);
+
 	qb_log(LOG_DEBUG, "Testing %d iterations of EVENT msg passing.", num_stress_events);
 
 	events_received = 0;
@@ -875,11 +1122,15 @@ test_ipc_stress_test(void)
 	qb_loop_run(cl);
 	ck_assert_int_eq(events_received, num_stress_events);
 
-	req_header.id = IPC_MSG_REQ_SERVER_FAIL;
-	req_header.size = sizeof(struct qb_ipc_request_header);
+	giant_req.hdr.id = IPC_MSG_REQ_SERVER_FAIL;
+	giant_req.hdr.size = sizeof(giant_req);
 
-	iov[0].iov_len = req_header.size;
-	iov[0].iov_base = &req_header;
+	if (giant_req.hdr.size <= client_buf_size) {
+		ck_assert_int_eq(1, 0);
+	}
+
+	iov[0].iov_len = giant_req.hdr.size;
+	iov[0].iov_base = &giant_req;
 	res = qb_ipcc_sendv_recv(conn, iov, 1,
 				 &res_header,
 				 sizeof(struct qb_ipc_response_header), -1);
@@ -889,7 +1140,7 @@ test_ipc_stress_test(void)
 	}
 
 	qb_ipcc_disconnect(conn);
-	stop_process(pid);
+	verify_graceful_stop(pid);
 }
 
 START_TEST(test_ipc_stress_test_us)
@@ -897,18 +1148,28 @@ START_TEST(test_ipc_stress_test_us)
 	qb_enter();
 	send_event_on_created = QB_FALSE;
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_stress_test();
 	qb_leave();
 }
 END_TEST
 
+START_TEST(test_ipc_stress_connections_us)
+{
+	qb_enter();
+	ipc_type = QB_IPC_SOCKET;
+	set_ipc_name(__func__);
+	test_ipc_stress_connections();
+	qb_leave();
+}
+END_TEST
+
 START_TEST(test_ipc_bulk_events_us)
 {
 	qb_enter();
 	send_event_on_created = QB_FALSE;
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_bulk_events();
 	qb_leave();
 }
@@ -923,6 +1184,7 @@ test_ipc_event_on_created(void)
 	int32_t res;
 	qb_loop_t *cl;
 	int32_t fd;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	num_bulk_events = 1;
 
@@ -931,7 +1193,7 @@ test_ipc_event_on_created(void)
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -953,8 +1215,9 @@ test_ipc_event_on_created(void)
 	qb_loop_run(cl);
 	ck_assert_int_eq(events_received, num_bulk_events);
 
+	request_server_exit();
 	qb_ipcc_disconnect(conn);
-	stop_process(pid);
+	verify_graceful_stop(pid);
 }
 
 START_TEST(test_ipc_event_on_created_us)
@@ -962,7 +1225,7 @@ START_TEST(test_ipc_event_on_created_us)
 	qb_enter();
 	send_event_on_created = QB_TRUE;
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_event_on_created();
 	qb_leave();
 }
@@ -978,13 +1241,14 @@ test_ipc_disconnect_after_created(void)
 	int32_t j = 0;
 	pid_t pid;
 	int32_t res;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	pid = run_function_in_new_process(run_ipc_server);
 	fail_if(pid == -1);
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -1015,7 +1279,7 @@ test_ipc_disconnect_after_created(void)
 	ck_assert_int_eq(QB_FALSE, qb_ipcc_is_connected(conn));
 
 	qb_ipcc_disconnect(conn);
-	stop_process(pid);
+	kill_server(pid);
 }
 
 START_TEST(test_ipc_disconnect_after_created_us)
@@ -1023,7 +1287,7 @@ START_TEST(test_ipc_disconnect_after_created_us)
 	qb_enter();
 	disconnect_after_created = QB_TRUE;
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_disconnect_after_created();
 	qb_leave();
 }
@@ -1032,20 +1296,17 @@ END_TEST
 static void
 test_ipc_server_fail(void)
 {
-	struct qb_ipc_request_header req_header;
-	struct qb_ipc_response_header res_header;
-	struct iovec iov[1];
-	int32_t res;
 	int32_t j;
 	int32_t c = 0;
 	pid_t pid;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	pid = run_function_in_new_process(run_ipc_server);
 	fail_if(pid == -1);
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -1055,38 +1316,17 @@ test_ipc_server_fail(void)
 	} while (conn == NULL && c < 5);
 	fail_if(conn == NULL);
 
-	/*
-	 * tell the server to exit
-	 */
-	req_header.id = IPC_MSG_REQ_SERVER_FAIL;
-	req_header.size = sizeof(struct qb_ipc_request_header);
-
-	iov[0].iov_len = req_header.size;
-	iov[0].iov_base = &req_header;
-
-	ck_assert_int_eq(QB_TRUE, qb_ipcc_is_connected(conn));
-
-	res = qb_ipcc_sendv_recv(conn, iov, 1,
-				 &res_header,
-				 sizeof(struct qb_ipc_response_header), -1);
-	/*
-	 * confirm we get -ENOTCONN or ECONNRESET
-	 */
-	if (res != -ECONNRESET && res != -ENOTCONN) {
-		qb_log(LOG_ERR, "id:%d size:%d", res_header.id, res_header.size);
-		ck_assert_int_eq(res, -ENOTCONN);
-	}
+	request_server_exit();
 	ck_assert_int_eq(QB_FALSE, qb_ipcc_is_connected(conn));
-
 	qb_ipcc_disconnect(conn);
-	stop_process(pid);
+	verify_graceful_stop(pid);
 }
 
 START_TEST(test_ipc_server_fail_soc)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_server_fail();
 	qb_leave();
 }
@@ -1096,7 +1336,7 @@ START_TEST(test_ipc_disp_shm)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_dispatch();
 	qb_leave();
 }
@@ -1107,17 +1347,27 @@ START_TEST(test_ipc_stress_test_shm)
 	qb_enter();
 	send_event_on_created = QB_FALSE;
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_stress_test();
 	qb_leave();
 }
 END_TEST
 
+START_TEST(test_ipc_stress_connections_shm)
+{
+	qb_enter();
+	ipc_type = QB_IPC_SHM;
+	set_ipc_name(__func__);
+	test_ipc_stress_connections();
+	qb_leave();
+}
+END_TEST
+
 START_TEST(test_ipc_bulk_events_shm)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_bulk_events();
 	qb_leave();
 }
@@ -1128,7 +1378,7 @@ START_TEST(test_ipc_event_on_created_shm)
 	qb_enter();
 	send_event_on_created = QB_TRUE;
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_event_on_created();
 	qb_leave();
 }
@@ -1138,7 +1388,7 @@ START_TEST(test_ipc_server_fail_shm)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_server_fail();
 	qb_leave();
 }
@@ -1150,6 +1400,7 @@ test_ipc_service_ref_count(void)
 	int32_t c = 0;
 	int32_t j = 0;
 	pid_t pid;
+	uint32_t max_size = MAX_MSG_SIZE;
 
 	reference_count_test = QB_TRUE;
 
@@ -1158,7 +1409,7 @@ test_ipc_service_ref_count(void)
 	sleep(1);
 
 	do {
-		conn = qb_ipcc_connect(ipc_name, MAX_MSG_SIZE);
+		conn = qb_ipcc_connect(ipc_name, max_size);
 		if (conn == NULL) {
 			j = waitpid(pid, NULL, WNOHANG);
 			ck_assert_int_eq(j, 0);
@@ -1170,8 +1421,7 @@ test_ipc_service_ref_count(void)
 
 	sleep(5);
 
-	qb_ipcc_disconnect(conn);
-	stop_process(pid);
+	kill_server(pid);
 }
 
 
@@ -1179,7 +1429,7 @@ START_TEST(test_ipc_service_ref_count_shm)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SHM;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_service_ref_count();
 	qb_leave();
 }
@@ -1189,7 +1439,7 @@ START_TEST(test_ipc_service_ref_count_us)
 {
 	qb_enter();
 	ipc_type = QB_IPC_SOCKET;
-	ipc_name = __func__;
+	set_ipc_name(__func__);
 	test_ipc_service_ref_count();
 	qb_leave();
 }
@@ -1232,24 +1482,29 @@ make_shm_suite(void)
 	TCase *tc;
 	Suite *s = suite_create("shm");
 
+	tc = tcase_create("ipc_txrx_shm_timeout");
+	tcase_add_test(tc, test_ipc_txrx_shm_timeout);
+	tcase_set_timeout(tc, 30);
+	suite_add_tcase(s, tc);
+
 	tc = tcase_create("ipc_server_fail_shm");
 	tcase_add_test(tc, test_ipc_server_fail_shm);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_txrx_shm_block");
 	tcase_add_test(tc, test_ipc_txrx_shm_block);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_txrx_shm_tmo");
 	tcase_add_test(tc, test_ipc_txrx_shm_tmo);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_fc_shm");
 	tcase_add_test(tc, test_ipc_fc_shm);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_dispatch_shm");
@@ -1269,11 +1524,12 @@ make_shm_suite(void)
 
 	tc = tcase_create("ipc_exit_shm");
 	tcase_add_test(tc, test_ipc_exit_shm);
-	tcase_set_timeout(tc, 3);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_event_on_created_shm");
 	tcase_add_test(tc, test_ipc_event_on_created_shm);
+	tcase_set_timeout(tc, 10);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_service_ref_count_shm");
@@ -1281,6 +1537,11 @@ make_shm_suite(void)
 	tcase_set_timeout(tc, 10);
 	suite_add_tcase(s, tc);
 
+	tc = tcase_create("ipc_stress_connections_shm");
+	tcase_add_test(tc, test_ipc_stress_connections_shm);
+	tcase_set_timeout(tc, 200);
+	suite_add_tcase(s, tc);
+
 	return s;
 }
 
@@ -1290,6 +1551,11 @@ make_soc_suite(void)
 	Suite *s = suite_create("socket");
 	TCase *tc;
 
+	tc = tcase_create("ipc_txrx_us_timeout");
+	tcase_add_test(tc, test_ipc_txrx_us_timeout);
+	tcase_set_timeout(tc, 30);
+	suite_add_tcase(s, tc);
+
 	tc = tcase_create("ipc_max_dgram_size");
 	tcase_add_test(tc, test_ipc_max_dgram_size);
 	tcase_set_timeout(tc, 30);
@@ -1297,27 +1563,27 @@ make_soc_suite(void)
 
 	tc = tcase_create("ipc_server_fail_soc");
 	tcase_add_test(tc, test_ipc_server_fail_soc);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_txrx_us_block");
 	tcase_add_test(tc, test_ipc_txrx_us_block);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_txrx_us_tmo");
 	tcase_add_test(tc, test_ipc_txrx_us_tmo);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_fc_us");
 	tcase_add_test(tc, test_ipc_fc_us);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_exit_us");
 	tcase_add_test(tc, test_ipc_exit_us);
-	tcase_set_timeout(tc, 6);
+	tcase_set_timeout(tc, 8);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_dispatch_us");
@@ -1337,10 +1603,12 @@ make_soc_suite(void)
 
 	tc = tcase_create("ipc_event_on_created_us");
 	tcase_add_test(tc, test_ipc_event_on_created_us);
+	tcase_set_timeout(tc, 10);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_disconnect_after_created_us");
 	tcase_add_test(tc, test_ipc_disconnect_after_created_us);
+	tcase_set_timeout(tc, 10);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ipc_service_ref_count_us");
@@ -1348,6 +1616,11 @@ make_soc_suite(void)
 	tcase_set_timeout(tc, 10);
 	suite_add_tcase(s, tc);
 
+	tc = tcase_create("ipc_stress_connections_us");
+	tcase_add_test(tc, test_ipc_stress_connections_us);
+	tcase_set_timeout(tc, 200);
+	suite_add_tcase(s, tc);
+
 	return s;
 }
 
@@ -1359,6 +1632,7 @@ main(void)
 	Suite *s;
 	int32_t do_shm_tests = QB_TRUE;
 
+	set_ipc_name("ipc_test");
 #ifdef DISABLE_IPC_SHM
 	do_shm_tests = QB_FALSE;
 #endif /* DISABLE_IPC_SHM */
