@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Red Hat, Inc.
+ * Copyright (c) 2011-2015 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -191,6 +191,7 @@ _test_logger(int32_t t,
 	test_buf[0] = '\0';
 	qb_log_target_format(t, cs, timestamp, msg, test_buf);
 	test_priority = cs->priority;
+
 	num_msgs++;
 }
 
@@ -282,6 +283,7 @@ START_TEST(test_log_basic)
 	rc = qb_log_ctl(t, QB_LOG_CONF_ENABLED, QB_TRUE);
 	ck_assert_int_eq(rc, 0);
 
+	/* captures last log */
 	memset(test_buf, 0, sizeof(test_buf));
 	test_priority = 0;
 	num_msgs = 0;
@@ -298,6 +300,54 @@ START_TEST(test_log_basic)
 	ck_assert_int_eq(num_msgs, 1);
 	ck_assert_str_eq(test_buf, "Hello Angus, how are you?");
 
+
+	/* 
+	 * test filtering by file regex
+ 	 */
+	qb_log_filter_ctl(t, QB_LOG_FILTER_CLEAR_ALL,
+			  QB_LOG_FILTER_FORMAT, "*", LOG_TRACE);
+	qb_log_filter_ctl(t, QB_LOG_FILTER_ADD,
+			  QB_LOG_FILTER_FILE_REGEX, "^fakefile*", LOG_DEBUG);
+
+	num_msgs = 0;
+	qb_log_from_external_source(__func__, "fakefile_logging", "%s bla", LOG_INFO,
+				    56, 0, "filename/lineno");
+	qb_log_from_external_source(__func__, "do_not_log_fakefile_logging", "%s bla", LOG_INFO,
+				    56, 0, "filename/lineno");
+	ck_assert_int_eq(num_msgs, 1);
+
+	/* 
+	 * test filtering by format regex
+ 	 */
+	qb_log_filter_ctl(t, QB_LOG_FILTER_CLEAR_ALL,
+			  QB_LOG_FILTER_FORMAT, "*", LOG_TRACE);
+	qb_log_filter_ctl(t, QB_LOG_FILTER_ADD,
+			  QB_LOG_FILTER_FORMAT_REGEX, "^one", LOG_WARNING);
+
+	num_msgs = 0;
+	qb_log(LOG_INFO, "one two three");
+	qb_log(LOG_ERR, "testing one two three");
+	qb_log(LOG_WARNING, "one two three");
+	qb_log(LOG_ERR, "one two three");
+	qb_log(LOG_EMERG, "one two three");
+	ck_assert_int_eq(num_msgs, 3);
+
+	/*
+	 * test filtering by function and regex
+	 */
+	qb_log_filter_ctl(t, QB_LOG_FILTER_CLEAR_ALL,
+			  QB_LOG_FILTER_FILE, "*", LOG_TRACE);
+	qb_log_filter_ctl(t, QB_LOG_FILTER_ADD,
+			  QB_LOG_FILTER_FUNCTION_REGEX, "^log_.*please", LOG_WARNING);
+
+	num_msgs = 0;
+	qb_log(LOG_ERR, "try if you: log_it_please()");
+	log_it_please();
+	ck_assert_int_eq(num_msgs, 3);
+
+	qb_log_filter_ctl(t, QB_LOG_FILTER_REMOVE,
+			  QB_LOG_FILTER_FUNCTION_REGEX, "log_it_please", LOG_WARNING);
+
 	/*
 	 * test filtering by function
 	 */
@@ -336,7 +386,7 @@ START_TEST(test_log_basic)
 	qb_log_filter_ctl(t, QB_LOG_FILTER_CLEAR_ALL,
 			  QB_LOG_FILTER_FILE, "*", LOG_TRACE);
 	qb_log_filter_ctl(t, QB_LOG_FILTER_ADD,
-			  QB_LOG_FILTER_FILE, __FILE__, LOG_DEBUG);
+			  QB_LOG_FILTER_FILE, "fakefile.c,"__FILE__",otherfakefile", LOG_DEBUG);
 	/*
 	 * make sure we can pass in a null filename or function name.
 	 */
@@ -658,6 +708,63 @@ START_TEST(test_threaded_logging)
 }
 END_TEST
 
+START_TEST(test_extended_information)
+{
+	int32_t t;
+	int32_t rc;
+	int extended;
+
+	qb_log_init("test", LOG_USER, LOG_DEBUG);
+	qb_log_ctl(QB_LOG_SYSLOG, QB_LOG_CONF_ENABLED, QB_FALSE);
+
+	t = qb_log_custom_open(_test_logger, NULL, NULL, NULL);
+	_ck_assert_int(t, >, QB_LOG_STDOUT);
+	qb_log_format_set(t, "%b");
+	rc = qb_log_filter_fn_set(NULL);
+	ck_assert_int_eq(rc, 0);
+	rc = qb_log_filter_ctl(t, QB_LOG_FILTER_CLEAR_ALL, QB_LOG_FILTER_FILE,
+			       "*", LOG_TRACE);
+	ck_assert_int_eq(rc, 0);
+	rc = qb_log_filter_ctl(t, QB_LOG_FILTER_CLEAR_ALL, QB_LOG_FILTER_FORMAT,
+			       "*", LOG_TRACE);
+	ck_assert_int_eq(rc, 0);
+	rc = qb_log_filter_ctl(t, QB_LOG_FILTER_ADD, QB_LOG_FILTER_FORMAT,
+			       "*", LOG_TRACE);
+	ck_assert_int_eq(rc, 0);
+	rc = qb_log_ctl(t, QB_LOG_CONF_ENABLED, QB_TRUE);
+	ck_assert_int_eq(rc, 0);
+
+	for (extended = QB_FALSE; extended <= QB_TRUE; ++extended) {
+		rc = qb_log_ctl(t, QB_LOG_CONF_EXTENDED, extended);
+		ck_assert_int_eq(rc, 0);
+
+		num_msgs = 0;
+
+		memset(test_buf, 0, sizeof(test_buf));
+		qb_log(LOG_ERR, "message with no extended information");
+		ck_assert_str_eq(test_buf, "message with no extended information");
+
+		memset(test_buf, 0, sizeof(test_buf));
+		qb_log(LOG_ERR, "message with empty extended information "QB_XS);
+		ck_assert_str_eq(test_buf, "message with empty extended information ");
+
+		memset(test_buf, 0, sizeof(test_buf));
+		qb_log(LOG_ERR, QB_XS" message with only extended information");
+		ck_assert_str_eq(test_buf, extended?
+				 "| message with only extended information" : "");
+
+		memset(test_buf, 0, sizeof(test_buf));
+		qb_log(LOG_ERR, "message with extended information "QB_XS" (namely this)");
+		ck_assert_str_eq(test_buf, extended?
+				 "message with extended information | (namely this)"
+				 : "message with extended information ");
+
+		ck_assert_int_eq(num_msgs, (extended? 4 : 3));
+	}
+	qb_log_fini();
+}
+END_TEST
+
 static Suite *
 log_suite(void)
 {
@@ -701,6 +808,10 @@ log_suite(void)
 	tcase_add_test(tc, test_threaded_logging);
 	suite_add_tcase(s, tc);
 
+	tc = tcase_create("extended_information");
+	tcase_add_test(tc, test_extended_information);
+	suite_add_tcase(s, tc);
+
 	return s;
 }
 
